from functools import wraps
import time
import sys
import os


def TypePrints(func):
    """Print func.__doc__ like type prints."""
    @wraps(func)
    def wrappers(*args, **kwargs):
        for c in func.__doc__:
            sys.stdout.write(c)
            sys.stdout.flush()
            time.sleep(0.08)
        sys.stdout.flush()
        sys.stdout.write('\n')
        return func(*args, **kwargs)
    return wrappers



from .Dtime import Counter, Timer, Retry_timer, Schedule
from .Dlog import Error_Log
from .Dfuns import TypePrints
__all__ = [
    'Counter',
    'Timer',
    'Retry_timer',
    'Schedule',
    'Error_Log',
    'TypePrints'
]



from functools import wraps
from .settings import logging

logger = logging.getLogger(__name__)


def Error_Log(func):
    """Decorator for logging Exception but not stop the program."""
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logger.warning(f'{func.__name__}:{e}', exc_info=True)
    return wrapper



"""Setup."""
from setuptools import setup, find_packages

if __name__ == '__main__':
    setup(
        name='yyutils',
        version='0.0.1',
        packages=find_packages(),
        author='yyh',
        author_email='hyuyu1544@gmail.com',
        python_requires='>=3.6',
        url='https://github.com/hyuyu1544/yutils',
    )



from functools import wraps
from .settings import logging
import time

logger = logging.getLogger(__name__)


def Counter(func):
    """Decorator for counting timer."""
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            ans = func(*args, **kwargs)
            logger.debug(
                f'Running func:{func.__name__} for {wrapper.calls} times.')
            wrapper.calls += 1
            return ans
        except Exception as e:
            logger.critical(e)
            logger.critical(f'{func.__name__} cannot work.')
    wrapper.calls = 1
    return wrapper


def Timer(func):
    """Decorator for timming timer."""
    @wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.perf_counter()
        ans = func(*args, **kwargs)
        end_time = time.perf_counter()
        running_time = end_time-start_time
        logger.debug(
            f'func:{func.__name__} running in {running_time:.4f} sec.')
        return ans
    return wrapper


def Retry_timer(interval=3, retry_times=3):
    """Decorator for retry func."""
    def retry_timer(func):
        @wraps(func)
        def wrapper(count=1, interval=interval, retry_times=retry_times, *args, **kwargs):
            try:
                logger.debug(f'Try func:{func.__name__} {count} times.')
                return func(*args, **kwargs)
            except Exception as e:
                logger.warning(f'There have some error: {e}')
                count += 1
                if count <= retry_times:
                    logger.debug(f'Will retry in {interval} sec.')
                    time.sleep(interval)
                    return wrapper(count=count, interval=interval, retry_times=retry_times, *args, **kwargs)
                else:
                    logger.critical(f'Failed to execute func:{func.__name__}')
        return wrapper
    return retry_timer


def Schedule(interval=3600):
    """Decorator for schedule program."""
    # TODO:: if func need to return something
    def schedule(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            while True:
                start_time = time.perf_counter()
                func(*args, **kwargs)
                running_time = time.perf_counter()-start_time
                latency = interval-running_time
                logger.debug(
                    f'Next program:{func.__name__} will start at {latency} sec later.')
                time.sleep(latency)
                return wrapper(*args, **kwargs)
        return wrapper
    return schedule
